'' =================================================================================================
''
''   File....... hackable_ebadge_opticspy_demo.spin
''   Purpose.... OpticSpy demo code for the Parallax Hackable Electronic Badge
''   Author..... Joe Grand, Grand Idea Studio [@joegrand]
''               -- see below for terms of use
''   Revisions.. 5 FEB 2018 (Initial release)
''
'' =================================================================================================

{

  This program uses the Parallax Electronic Hackable Badge (#20000) to create an optical
  covert channel. Optical covert channels transmit data in a way that is undetectable to
  the human eye. Typically achieved with visible light, they can also be done with infrared
  LEDs available on a device. This program sends ASCII printable data via the IR-OUT LED
  using a standard UART interface (in this case, the jm_fullduplexserial object).

  More details about this example and the hardware needed to receive the data can be found at:
  http://www.grandideastudio.com/portfolio/optical-covert-channels/

  Code based on Parallax's Open Hardware Summit 2015 Badge written by Jon "JonnyMac" McPhalen:
  https://www.parallax.com/downloads/hackable-electronic-badge-ohs-spin-code
  
}

                 
con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us
                 
  T_BAUD   = 9600  { baud rate for data transmission } 
  

con { io pins }

  RX1      = 31             ' programming / terminal
  TX1      = 30

  SDA      = 29             ' eeprom / i2c
  SCL      = 28
     
  IR_OUT   = 24             ' IR communications
  IR_IN    = 23
  
  OLED_DAT = 22             ' OLED connections 
  OLED_CLK = 21
  OLED_DC  = 20  
  OLED_RST = 19  
  OLED_CS  = 18
  

con

  #0, NO, YES
                                                    
  #0, MSG_NAME              ' display messages
  #1, MSG_COVERT

  NAME_CHARS  = 8           ' using 2x8 Parallax font mode in OLED
  SCROLL_MS   = 250         ' scroll speed
                 
  
obj

' main                                  ' * master Spin cog
  time   : "jm_time"                    ' timing and delays
  dtmr   : "jm_time"                    ' display state timer
  scrtmr : "jm_time"                    ' scroll timer  
  io     : "jm_io"                      ' io control
  oled   : "oled_asmfast"               ' * OLED driver
  term   : "jm_fullduplexserial"        ' * full-duplex serial

' * Uses cog when loaded                                          

var

  long  dstate                          ' display state
  long  msgms                           ' message/display milliseconds

  long  scrollidx                       ' scroll position
  long  scrollidxmax
  byte  scroll[32]                      ' scroll buffer
  
  
dat { messages }

' Non-scrolling messages
' -- Lines should be limited to 8 characters
'    * will be truncated if longer 
 
  MsgName_0        byte    "OpticSpy", 0 
  MsgName_1        byte    "  Demo  ", 0  

  MsgCovert_0      byte    "Sending:", 0
  
' Specify scrolling lines without any space padding
' -- maximum length of either name is 24
  MsgCovert_1      byte    "Secret Message!", 0  

    
pub main | t

  setup                                       ' setup io and badge objects

  dstate := MSG_NAME                          ' show start-up message on LCD
  update_display(1000)
  time.pause(1500)                            ' additional delay               

  dstate := MSG_COVERT
  update_display(1000)
  time.pause(250)                             ' additional delay               
  
  repeat  
    check_scroll                              ' update scrolling message on LCD
    term.str(@MsgCovert_1)                    ' transmit secret message via LED
    term.tx(term#CR)  
    time.pause(25)                            ' delay to let receiver properly synchronize

    
pub check_scroll | len

'' Scroll MsgCovert_1 message along bottom line of display

  if (scrtmr.millis => 0)                                            ' if delay done             
    if (++scrollidx > scrollidxmax)                                  ' at end of scroll window?                                                      
      scrollidx := -NAME_CHARS                                             '  reset
                                                                     
    bytefill(@scroll, " ", 32)                                       ' fill scroll buf with spaces
    len := strsize(@MsgCovert_1)                                     ' get length of segment     
    if (scrollidx < 0)                                               ' scroll on?
      bytemove(@scroll + ||scrollidx, @MsgCovert_1, len)                   ' put text on right side of display
    else                                                             ' scroll off
      if (scrollidx < len)                                                 ' anything viewable for this index?
        bytemove(@scroll, @MsgCovert_1 + scrollidx, len-scrollidx)           ' display current segment                                     
    oled.write2x8String(@scroll, 8, 1)                               ' write in oled                       

    oled.updateDisplay                                         ' refresh display
    scrtmr.set(-SCROLL_MS)                                     ' reset scrolling timer
          

pub update_display(ms) | p_str0, p_str1

'' Displays "screen" based on dstate
'' -- ms is milliseconds to display message   

  case dstate
    MSG_NAME:
      p_str0 := @MsgName_0
      p_str1 := @MsgName_1 

    MSG_COVERT:
      p_str0 := @MsgCovert_0
      p_str1 := @MsgCovert_1
    
  if (dstate => 0)
    'oled.clearDisplay
    bytefill(oled.getBuffer, 0, 1024)
    oled.write2x8String(p_str0, strsize(p_str0) <# 8, 0)  
    oled.write2x8String(p_str1, strsize(p_str1) <# 8, 1)

  oled.updateDisplay

  msgms := ms                                                    ' set message time
  dtmr.start                                                     ' restart display timer


pub setup                              

'' Setup IO and objects for application

  time.start                                                     ' setup timing/delays for clock speed
  dtmr.start
  scrtmr.start

  io.start($0000, $0000)                                         ' clear all pins (this cog)

  oled.init(OLED_CS, OLED_DC, OLED_DAT, OLED_CLK, OLED_RST, {    ' start buffered oled driver
}           oled#SSD1306_SWITCHCAPVCC, oled#TYPE_128X64)
  oled.AutoUpdateOff  
  
  term.start(RX1, IR_OUT, %0010, T_BAUD)                         ' start serial

  if (strsize(@MsgCovert_1) > 24)
    byte[@MsgCovert_1][24] := 0                                  ' truncate if needed

  scrollidxmax :=  strsize(@MsgCovert_1)                         ' find max idx for scrolling


dat { license }

{{

  Copyright (C) 2018 Grand Idea Studio, Inc.

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}